@@ -27,6 +27,11 @@ MESSAGEMANAGER_FUZZER_STRINGSFILE=<path> (optional)
 MESSAGEMANAGER_FUZZER_BLACKLIST=<path> (optional)
 */
 
+#ifdef IsLoggingEnabled
+// This is defined in the Windows SDK urlmon.h
+#  undef IsLoggingEnabled
+#endif
+
 class MessageManagerFuzzer {
  public:
   static void TryMutate(JSContext* aCx, const nsAString& aMessageName,
